/*********************************************************************
 *
 * File:         message_send.c
 *
 * Description:  Messages send module for conference engine
 *
 * Author:       Tomasz Stachowiak, NPAC
 *
 * Date:         10-15-96
 *
 *********************************************************************/
#include <stdio.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>

#include <netdb.h>
#include <unistd.h>

#include "global.h"

int sendControlMessage(char local, int user, short tag, char *data, int len)
{
  void *buf;
  Message *msg;
  int size;
  struct sockaddr_in addr;
  int i;
  char userid[20];
  char hostname[50];
  int sent = 0;

  if(data != NULL)
    {
      size = sizeof(Message) + len - 1;
      buf = malloc(size);
      if(buf == NULL)
	fprintf(stderr, "Engine: control message allocation error\n");
      msg = (Message *)buf;
      memcpy(msg->data, data, len);
    }
  else
    {
      size = sizeof(Message);
      buf = malloc(size);
      if(buf == NULL)
	fprintf(stderr, "Engine: control message allocation error\n");
      msg = (Message *)buf;
      msg->data[0] = 0;
    }

  msg->tag = tag;
  cuserid(userid);
  gethostname(hostname, 50);
  strcpy(msg->sender.user, userid);
  strcpy(msg->sender.host, hostname);

  if(local == LOCAL)
    {
      PRINTF("Engine: message local send, tag %d\n", msg->tag);
      msg->local = LOCAL;
      getManagerAddr(&addr);
      if(sendControlPacket(&addr, (char *)msg, size) < 0)
	{
	  free(msg);
	  return 0; 
	}
    }
  else
    {
      msg->local = REMOTE;
      if(user >= 0)
	{
	  if(getUserData(user, &addr, &msg->receiver))
	    if(sendControlPacket(&addr, (char *)msg, size) < 0)
	      {
		free(msg);
		return 0;
	      } 

	  /* if message to initiator find initiator */
	  if((user == 0) && (!getUserData(user, &addr, &msg->receiver)))
	    {
	      i = 0;
	      while((i < MAX_MEMBERS) && (!sent))
		{
		  if(getUserData(i, &addr, &msg->receiver))
		    {
		      if(sendControlPacket(&addr, (char *)msg, size) < 0)
			{
			  free(msg);
			  return 0;
			} 
		      sent = 1;
		    }
		  i++;
		}
	    }
		  
		
	}
      else
	{
	  for(i=0; i<MAX_MEMBERS; i++)
	    {
	      if(getUserData(i, &addr, &msg->receiver))
		{
		  if(sendControlPacket(&addr, (char *)msg, size) < 0)
		    {		    
		      free(msg);
		      return 0; 
		    }
		}
	    }
	}
    }

  free(msg);
  return 1;
}

int sendControlMessageToUser(Participant *part, short tag, 
			     char *data, int len)
{
  Message *msg;
  struct hostent *hostinfo;
  struct sockaddr_in addr;
  int size;

  PRINTF("Engine: send message to %s@%s \n", part->user, part->host);
  hostinfo = gethostbyname(part->host);
  if(hostinfo == NULL)
    {
      fprintf(stderr, "Engine: send message unknown host %s\n", part->host);
      return 0;
    }

  if(data != NULL)
    {
      size = sizeof(Message) + len - 1;
      msg = malloc(size);
      if(msg == NULL)
	fprintf(stderr, "Engine: control message allocation error\n");
      memcpy(msg->data, data, len);
    }
  else
    {
      size = sizeof(Message);
      msg = malloc(size);
      if(msg == NULL)
	fprintf(stderr, "Engine: control message allocation error\n");
      msg->data[0] = 0;
    }


  msg->local = REMOTE;
  msg->tag = tag;
  cuserid(msg->sender.user);
  gethostname(msg->sender.host, 50);
  strcpy(msg->receiver.user, part->user);
  strcpy(msg->receiver.host, part->host);

  bzero(&addr, sizeof(struct sockaddr_in));
  addr.sin_family = AF_INET;
  addr.sin_addr.s_addr = ((struct in_addr *)hostinfo->h_addr)->s_addr;
  addr.sin_port = htons(DAEMON_PORT);

  if(sendControlPacket(&addr, (char *)msg, size) < 0)
    {
      free(msg);
      return 0; 
    }

  free(msg);
  return 1;
}




